@@ -10,6 +10,7 @@ def load_process_data(
     dark_filename: Optional[str] = None,
     copy_files: bool = True,
     isH5: bool = False,
+    title: str = "",
 ):
     """When `filenames` is a string, it will be treated as a file pattern."""
     indices = li_indices = None
@@ -24,6 +25,7 @@ def load_process_data(
             in_memory=in_memory,
             copy_files=copy_files,
             isH5=isH5,
+            title=title
         )
     else:
         filenames = list(filenames)
@@ -35,6 +37,7 @@ def load_process_data(
             in_memory=in_memory,
             copy_files=copy_files,
             isH5=isH5,
+            title=title
         )
 
     if dark_filename:
