@@ -216,7 +216,7 @@ def threshold_removal(data, bottom=None, top=None):
     """
     Set bottom and top threshold to the images in the dataset.
 
-    :param Dataset dataset: Dataset with the data
+    :param array_like data: Input data
     :param int bottom: Bottom threshold
     :param int top: Top threshold
     :returns: ndarray
@@ -229,3 +229,18 @@ def threshold_removal(data, bottom=None, top=None):
         new_data[new_data > top] = 0
 
     return new_data
+
+
+def mask_removal(data, mask):
+    """
+    Set 0 values of mask to 0.
+
+    :param array_like data: Input data
+    :param nd.array mask: Input mask.
+
+    :returns: ndarray with the masked values
+    """
+    new_data = numpy.array(data, dtype=data.dtype)
+    mask = numpy.array(mask)
+
+    return new_data * mask
