@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "06/12/2019"
+__date__ = "24/09/2020"
 
 
 import numpy
@@ -54,7 +54,8 @@ def apply_2D_ROI(img, origin=None, size=None, center=None):
             and all(j < img.shape[i] for i, j in enumerate(origin)), "Origin must be a valid pixel"
         origin = numpy.array(origin)
         size = numpy.array(size)
-        points = numpy.array([origin, origin + size])
+        points = numpy.ceil([origin, origin + size]).astype(numpy.int)
+        points[1] = numpy.minimum(points[1], img.shape)
     elif center is not None:
         assert all(i >= 0 for i in center) \
             and all(j < img.shape[i] for i, j in enumerate(center)), "Center must be a valid pixel"
@@ -82,7 +83,6 @@ def apply_3D_ROI(data, origin=None, size=None, center=None):
     :returns: ndarray
     :raises: AssertionError, ValueError
     """
-
     assert size is not None, "The size of the roi must be given"
 
     data = numpy.asanyarray(data)
@@ -92,7 +92,8 @@ def apply_3D_ROI(data, origin=None, size=None, center=None):
             and all(j < data[0].shape[i] for i, j in enumerate(origin)), "Origin must be a valid pixel"
         origin = numpy.array(origin)
         size = numpy.array(size)
-        points = numpy.array([origin, origin + size])
+        points = numpy.ceil([origin, origin + size]).astype(numpy.int)
+        points[1] = numpy.minimum(points[1], data[0].shape)
     elif center is not None:
         assert all(i >= 0 for i in center) \
             and all(j < data[0].shape[i] for i, j in enumerate(center)), "Center must be a valid pixel"
