@@ -98,9 +98,9 @@ class TestDataset(unittest.TestCase):
         idxs = [0, 2, 4]
         data[idxs] += background[idxs]
         dataset = utils.createDataset(data=data, _dir=self._dir)
-        hi_idx, li_idx = dataset.partition_by_intensity()
-        self.assertEqual(hi_idx.shape[0], 3)
-        self.assertEqual(li_idx.shape[0], 7)
+        used_idx, not_used_idx = dataset.partition_by_intensity(bottom_bin=1)
+        self.assertEqual(used_idx.shape[0], 3)
+        self.assertEqual(not_used_idx.shape[0], 7)
 
     def test_bs_in_memory(self):
         """ Tests the background subtraction function with data in memory"""
@@ -333,11 +333,10 @@ class TestDataset(unittest.TestCase):
         dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10, in_memory=False)
         roi_dataset = dataset.apply_roi(origin=[0, 0], size=[20, 20])
         indices = [0, 2, 4, 5, 7]
-        bs_dataset = roi_dataset.apply_hot_pixel_removal(indices=indices)
-
-        self.assertEqual(bs_dataset.nframes, dataset.nframes)
-        self.assertNotEqual(dataset.data.urls[0], bs_dataset.data.urls[0])
-        numpy.testing.assert_equal(bs_dataset.data[6], roi_dataset.data[6])
+        hp_dataset = roi_dataset.apply_hot_pixel_removal(indices=indices)
+        self.assertEqual(hp_dataset.nframes, dataset.nframes)
+        self.assertNotEqual(dataset.data.urls[0], hp_dataset.data.urls[0])
+        numpy.testing.assert_equal(hp_dataset.data[6], roi_dataset.data[6])
 
     def test_roi_threshold_in_memory(self):
         """ Tests the threshold removal function with data in memory"""
@@ -482,23 +481,25 @@ class TestDataset(unittest.TestCase):
 
     def test_apply_fit_in_memory(self):
         """ Tests fitting data in memory"""
-        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10,
+        dataset = utils.createRandomDataset(dims=(10, 100), nb_data_files=10,
                                             header=True, _dir=self._dir)
-        new_dataset = dataset.apply_fit()
+        new_dataset, maps = dataset.apply_fit()
 
         self.assertEqual(new_dataset.nframes, dataset.nframes)
         self.assertNotEqual(new_dataset.data.urls[0], self.dataset.data.urls[0])
+        self.assertEqual(len(maps), 4)
 
     def test_apply_fit_in_disk(self):
         """Tests fitting data in disk"""
 
-        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10, in_memory=False)
+        dataset = utils.createRandomDataset(dims=(10, 100), nb_data_files=10, in_memory=False)
         indices = [0, 1, 2, 3, 4]
-        fit_dataset = dataset.apply_fit(indices=indices)
+        fit_dataset, maps = dataset.apply_fit(indices=indices)
 
         self.assertEqual(fit_dataset.nframes, dataset.nframes)
         self.assertNotEqual(fit_dataset.data.urls[0], dataset.data.urls[0])
         numpy.testing.assert_equal(fit_dataset.data[5], dataset.data[5])
+        self.assertEqual(len(maps), 4)
 
     def test_compute_rsm_in_memory(self):
         """ Tests fitting data in memory"""
@@ -614,6 +615,13 @@ class TestData(unittest.TestCase):
 
         numpy.testing.assert_array_equal(data.shape, (10, 100, 100))
 
+    def test_save_new_shape(self):
+        data = numpy.random.rand(10, 50, 50).view(Data)
+        # data.in_memory = True
+        data.save(os.path.join(self._dir, "data.hdf5"), new_shape=(10, 50, 50))
+
+        numpy.testing.assert_array_equal(data.shape, (10, 50, 50))
+
     def test_convert_to_hdf5(self):
         """ Tests the reshape of the data"""
         data = Data(urls=self.urls, metadata=self.metadata)
