@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "01/04/2020"
+__date__ = "08/12/2020"
 
 
 import unittest
@@ -92,13 +92,13 @@ class TestImageOperations(unittest.TestCase):
 
         chunk_shape = [0, 0]
 
-        img = imageOperations.chunk_image(self.data[0], start, chunk_shape)
+        img = imageOperations.chunk_image(start, chunk_shape, self.data[0])
 
         self.assertEqual(0, img.size)
 
         chunk_shape = [2, 2]
 
-        img = imageOperations.chunk_image(self.data[0], start, chunk_shape)
+        img = imageOperations.chunk_image(start, chunk_shape, self.data[0])
 
         numpy.testing.assert_array_equal(self.data[0, :2, :2], img)
 
@@ -161,12 +161,12 @@ class TestImageOperations(unittest.TestCase):
     def test_threshold_removal(self):
         """ Tests the threshold of the data"""
 
-        expected = numpy.array([[[1, 2, 3, 4, 3], [2, 2, 3, 4, 3], [3, 2, 3, 4, 3],
-                                 [1, 2, 3, 4, 3], [1, 2, 3, 4, 3]],
-                                [[1, 2, 3, 4, 3], [1, 2, 3, 4, 3], [1, 2, 3, 4, 3],
-                                 [1, 2, 3, 4, 3], [1, 2, 3, 4, 3]],
-                                [[1, 2, 3, 4, 3], [1, 2, 3, 4, 3], [1, 2, 3, 4, 3],
-                                 [3, 2, 3, 4, 3], [1, 2, 3, 4, 3]]])
+        expected = numpy.array([[[1, 2, 3, 4, 0], [2, 2, 3, 4, 0], [3, 2, 3, 4, 0],
+                                 [1, 2, 3, 4, 0], [1, 2, 3, 4, 0]],
+                                [[1, 2, 3, 4, 0], [1, 2, 3, 4, 0], [1, 2, 3, 4, 3],
+                                 [1, 2, 3, 4, 0], [1, 2, 3, 4, 0]],
+                                [[1, 2, 3, 4, 0], [1, 2, 3, 4, 0], [1, 2, 3, 4, 0],
+                                 [0, 2, 3, 4, 0], [1, 2, 3, 4, 0]]])
 
         data = imageOperations.threshold_removal(self.data, 1, 4)
 
