@@ -67,19 +67,22 @@ class TestMapping(unittest.TestCase):
 
         y, bins = numpy.histogram(samples, bins=100)
 
-        y_pred = mapping.fit_rocking_curve([y, None])
+        y_pred, pars = mapping.fit_rocking_curve([y, None])
         rss = numpy.sum((y - y_pred)**2)
         tss = numpy.sum((y - y.mean())**2)
         r2 = 1 - rss / tss
 
         self.assertGreater(r2, 0.9)
+        self.assertEqual(len(pars), 4)
 
     def test_fit_data(self):
         """ Tests the new data has same shape as initial data"""
 
-        new_data = mapping.fit_data(self.data)
+        new_data, maps = mapping.fit_data(self.data)
 
         self.assertEqual(new_data.shape, self.data.shape)
+        self.assertEqual(len(maps), 4)
+        self.assertEqual(maps[0].shape, self.data[0].shape)
 
     def test_moments(self):
         """ Tests the correct moments calculation """
