@@ -56,7 +56,7 @@ class Base():
         self._num_samples, self._num_features = self.data.shape
         if indices is not None:
             self._num_samples = len(indices)
-        self._indices = sorted(indices) if indices is not None else None
+        self._indices = sorted(indices) if indices is not None else numpy.arange(len(data))
         self._num_components = (num_components if num_components
                                 else min(self.num_samples, self.num_features))
 
@@ -115,10 +115,10 @@ class Base():
             err = 0
             for i in range(0, self.data.shape[1], chunks):
                 if scipy.sparse.issparse(self.data):
-                    tmp = self.data[:, i:i + chunks] - (self.W * self.H[:, i:i + chunks])
+                    tmp = self.data[self.indices, i:i + chunks] - (self.W * self.H[:, i:i + chunks])
                     err = tmp.multiply(tmp).sum()
                 else:
-                    err += numpy.sum((self.data[:, i:i + chunks] - numpy.dot(self.W, self.H[:, i:i + chunks]))**2)
+                    err += numpy.sum((self.data[self.indices, i:i + chunks] - numpy.dot(self.W, self.H[:, i:i + chunks]))**2)
             err = numpy.sqrt(err)
         else:
             err = None
@@ -138,10 +138,10 @@ class Base():
             err = 0
             for i in range(0, self.data.shape[1], chunks):
                 if scipy.sparse.issparse(self.data):
-                    tmp = self.data[:, i:i + chunks] - (self.W * self.H[:, i:i + chunks])
+                    tmp = self.data[self.indices, i:i + chunks] - (self.W * self.H[:, i:i + chunks])
                     err = tmp.multiply(tmp).sum()
                 else:
-                    err += numpy.sum((self.data[:, i:i + chunks] - numpy.dot(self.W, self.H[:, i:i + chunks]))**2)
+                    err += numpy.sum((self.data[self.indices, i:i + chunks] - numpy.dot(self.W, self.H[:, i:i + chunks]))**2)
         else:
             err = None
         return err
