@@ -255,6 +255,14 @@ class DimensionWidget(DimensionMapping):
         self._update_dataset = dataset
         self._parent._updateDataset(widget, dataset)
 
+    def _showSnowMessage(self):
+        msg = qt.QMessageBox()
+        msg.setIcon(qt.QMessageBox.Critical)
+        msg.setText("Dimension definition is not yet working with hdf5 files")
+        msg.setWindowTitle("Don't eat yellow snow!")
+        msg.setStandardButtons(qt.QMessageBox.Ok)
+        msg.exec_()
+
     def _find_dimensions(self):
         """
         Automatically find all dimensions that change through the dataset.
@@ -263,7 +271,9 @@ class DimensionWidget(DimensionMapping):
         to compare, and the threshold to know when two values are considered to
         be different. The threshold is only used in values that are numbers.
         """
-        self.dataset.find_dimensions(self.metadata_type, self.tolerance)
+        result = self.dataset.find_dimensions(self.metadata_type, self.tolerance)
+        if not result:
+            self._showSnowMessage()
         super().clear()
         for dim in self.dataset.dims:
             self.addDim(dim[0], dim[1])
@@ -342,14 +352,17 @@ class DimensionWidget(DimensionMapping):
         """
         self.clear()
         assert type(dims) is dict
+        if self.dataset._isH5:
+            self._showSnowMessage()
+            return
+
         for axis, dim in dims.items():
             assert type(axis) is int
             assert isinstance(dim, Dimension)
             self.addDim(axis=axis, dim=dim)
             if self.dataset is not None and len(self.dataset.get_data().metadata) > 0:
                 if not dim.unique_values:
-                    values = [data.get_value(kind=dim.kind, name=dim.name)[0]
-                              for data in self.dataset.get_data().metadata]
+                    values = self.dataset.get_metadata_values(dim.kind, dim.name)
                     dim.set_unique_values(values)
             self.dataset.add_dim(axis=axis, dim=dim)
 
@@ -477,8 +490,8 @@ class _DimensionItem(Dimension, qt.QWidget):
             self._maxLE.setText(str(numpy.round(_range[1], 5)))
             self._stepLE.setText(str(numpy.round(_range[2], 5)))
             self._minLE.setCursorPosition(0)
-            self._minLE.setCursorPosition(0)
-            self._minLE.setCursorPosition(0)
+            self._maxLE.setCursorPosition(0)
+            self._stepLE.setCursorPosition(0)
 
     def __init__(self, parent, table, row):
         """
