@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "26/04/2021"
+__date__ = "12/04/2022"
 
 import numpy
 
@@ -34,16 +34,16 @@ from silx.gui import qt
 from silx.gui import icons
 from silx.gui.colors import Colormap
 from silx.gui.plot import ScatterView, StackView, Plot1D
-from silx.gui.widgets.FrameBrowser import HorizontalSliderWithBrowser
 import silx.resources
 
 import darfix
 from darfix.io.utils import write_components
+from .utils import ChooseDimensionDock
 
 
 class DisplayComponentsWidget(qt.QMainWindow):
     """
-    Widget to apply blind source separation.
+    Widget to display rocking curves and scattering plot from a set of components
     """
 
     def __init__(self, parent=None):
@@ -84,7 +84,7 @@ class DisplayComponentsWidget(qt.QMainWindow):
     def showButton(self):
         self._computeB.show()
 
-    def setComponents(self, components, W, dimensions, shape):
+    def setComponents(self, components, W, dimensions, values, title=""):
         """
         Components setter. Updates the plots with the components and their
         corresponding rocking curves.
@@ -95,10 +95,12 @@ class DisplayComponentsWidget(qt.QMainWindow):
         """
         self.components = numpy.round(components, 5)
         self.W = W
+        self.values = values
         self.dimensions = dimensions
-        self.dimensions_shape = shape
         self._sv_components.setStack(self.components)
-        self._mixing_plots_widget.updatePlots(self.dimensions, self.W.T, self.dimensions_shape)
+        if title != "":
+            self._sv_components.setTitleCallback(lambda idx: title)
+        self._mixing_plots_widget.updatePlots(self.dimensions, self.W.T, values)
 
     def _saveComp(self):
         """
@@ -112,7 +114,8 @@ class DisplayComponentsWidget(qt.QMainWindow):
         fileDialog.setDefaultSuffix(".h5")
         if fileDialog.exec_():
             write_components(fileDialog.selectedFiles()[0], 'entry',
-                             self.dimensions, self.W, self.dimensions_shape, self.components, 1)
+                             self.dimensions.to_dict(), self.W, self.components,
+                             self.values, 1)
 
     def _setComponent(self, index=None):
         if index is not None:
@@ -153,7 +156,7 @@ class MixingPlotsWidget(qt.QMainWindow):
         self._is2dimensional = False
 
         self._plot_rocking_curves = PlotRockingCurves()
-        self._plot_rocking_curves.sigFrameChanged.connect(self._updateFrameNumber)
+        self._plot_rocking_curves.sigStateDisabled.connect(self._wholeStack)
         self._plot_rocking_curves.sigMotorChanged.connect(self._updateMotorAxis)
         self._plot_rocking_curves.sigActiveCurveChanged.connect(self._activeCurveChanged)
 
@@ -193,42 +196,38 @@ class MixingPlotsWidget(qt.QMainWindow):
     def scatter(self):
         return self._rsm_scatter
 
-    def updatePlots(self, dimensions, matrix, shape):
+    def updatePlots(self, dimensions, mixing_matrix, dimensions_values):
 
         self.dimensions = dimensions
-        self.matrix = matrix
-        self.dimensions_shape = shape
-        self._plot_rocking_curves.showMotors(True if len(self.dimensions) > 1 else False)
-        self.is2dimensional(True if len(dimensions) == 2 else False)
+        self.mixing_matrix = mixing_matrix
+        self.is2dimensional(True if dimensions.ndim == 2 else False)
         self._activateCurvesPlot(True)
         self.clearPlots()
+        self.values = numpy.array(list(dimensions_values.values()))
 
-        for i, row in enumerate(self.matrix):
+        for i, row in enumerate(self.mixing_matrix):
             self._plot_rocking_curves.getPlot().addCurve(numpy.arange(len(row)),
                                                          row, legend=str(i))
 
         if self._is2dimensional:
             colormap = Colormap(name='jet', normalization='linear')
-            keys = list(self.dimensions.keys())
-            self._rsm_scatter.getPlotWidget().setGraphXLabel(keys[0])
-            self._rsm_scatter.getPlotWidget().setGraphYLabel(keys[1])
-            self._rsm_scatter.setData(
-                numpy.asanyarray(self.dimensions[keys[0]], dtype=float),
-                numpy.asanyarray(self.dimensions[keys[1]], dtype=float), self.matrix[0])
+            self._rsm_scatter.getPlotWidget().setGraphXLabel(self.dimensions.get(0).name)
+            self._rsm_scatter.getPlotWidget().setGraphYLabel(self.dimensions.get(1).name)
+            self._rsm_scatter.setData(self.values[0], self.values[1], self.mixing_matrix[0])
             self._rsm_scatter.setColormap(colormap)
             self._rsm_scatter.resetZoom()
             self._toolbar.show()
         else:
             self._toolbar.hide()
 
-        if len(self.dimensions_shape) > 1:
-            self._plot_rocking_curves.showMotors(True)
-            self._plot_rocking_curves.resetDimensions()
+        if self.dimensions.ndim > 1:
             # Motor dimension and index
             self.dimension = 0
             self.index = 0
-            for dimension in self.dimensions:
-                self._plot_rocking_curves.addDimension(dimension)
+            self._plot_rocking_curves._chooseDimensionDock.show()
+            self._plot_rocking_curves._chooseDimensionDock.widget.setDimensions(self.dimensions)
+        else:
+            self._plot_rocking_curves._chooseDimensionDock.hide()
 
         self._plot_rocking_curves.getPlot().setActiveCurve("0")
 
@@ -244,18 +243,16 @@ class MixingPlotsWidget(qt.QMainWindow):
     def _activeCurveChanged(self, prev_legend=None, legend=None):
         if legend:
             self.activeCurveChanged.emit(int(legend))
-            values = numpy.array(list(self.dimensions.values())).astype(numpy.float)
             if self._is2dimensional:
-                self._rsm_scatter.setData(values[0], values[1], self.matrix[int(legend)])
+                self._rsm_scatter.setData(self.values[0], self.values[1], self.mixing_matrix[int(legend)])
 
     def _setCurve(self, index=-1):
         if index >= 0:
             status = self._plot_rocking_curves.blockSignals(True)
             self._plot_rocking_curves.getPlot().setActiveCurve(str(index))
             self._plot_rocking_curves.blockSignals(status)
-            values = numpy.array(list(self.dimensions.values())).astype(numpy.float)
             if self._is2dimensional:
-                self._rsm_scatter.setData(values[0], values[1], self.matrix[index])
+                self._rsm_scatter.setData(self.values[0], self.values[1], self.mixing_matrix[index])
 
     def _updateFrameNumber(self, index):
         """Update the current plot.
@@ -264,43 +261,50 @@ class MixingPlotsWidget(qt.QMainWindow):
         """
         self.index = index
         self.clearPlots()
-        dim_values = numpy.array(list(self.dimensions.values())[self.dimension])
-        dim_values = numpy.take(dim_values.reshape(self.dimensions_shape), self.index, self.dimension)
-        for i, row in enumerate(self.matrix):
-            W = numpy.take(row.reshape(self.dimensions_shape), self.index, self.dimension)
+        dim_values = numpy.take(self.values.reshape(self.dimensions.shape), self.index, self.dimension)
+        for i, row in enumerate(self.mixing_matrix):
+            W = numpy.take(row.reshape(self.dimensions.shape), self.index, self.dimension)
             self._plot_rocking_curves.addCurve(dim_values, W, legend=str(i))
 
-    def _updateMotorAxis(self, axis=-1):
+    def _wholeStack(self):
+        # Whole rocking curves are showed
+        self._plot_rocking_curves.clear()
+        for i, row in enumerate(self.mixing_matrix):
+            self._plot_rocking_curves.addCurve(numpy.arange(len(row)), row, legend=str(i))
+        self._plot_rocking_curves.getPlot().setGraphXLabel("Image id")
+
+    def _updateMotorAxis(self, dim=0, val=0):
         """
         Updates the motor to show the rocking curve from.
 
         :param int axis: 0 if no motor is chosen, else axis of the motor.
         """
-        if axis == 0:
-            # Whole rocking curves are showed
-            self._plot_rocking_curves.clear()
-            for i, row in enumerate(self.matrix):
-                self._plot_rocking_curves.addCurve(numpy.arange(len(row)), row, legend=str(i))
-            self._plot_rocking_curves.getBrowser().setEnabled(False)
-            self._plot_rocking_curves.getPlot().setGraphXLabel("Image id")
-        elif axis != -1 and axis is not None:
-            self.dimension = len(self.dimensions) - axis
-            self._plot_rocking_curves.clear()
-            self._plot_rocking_curves.getBrowser().setEnabled(True)
-            self._plot_rocking_curves.getBrowser().setRange(0, self.dimensions_shape[self.dimension] - 1)
-            self._plot_rocking_curves.getBrowser().setValue(0)
-            text = list(self.dimensions.keys())[self.dimension]
-            self._plot_rocking_curves.getPlot().setGraphXLabel(text)
-            dim_values = numpy.array(list(self.dimensions.values())[self.dimension])
-            dim_values = numpy.take(dim_values.reshape(self.dimensions_shape), self.index, self.dimension)
-            for i, row in enumerate(self.matrix):
-                W = numpy.take(row.reshape(self.dimensions_shape), self.index, self.dimension)
-                self._plot_rocking_curves.addCurve(dim_values, W, legend=str(i))
-        else:
-            raise ValueError('Axis %s is not managed' % axis)
+        dimension = [dim, val]
+        # Make sure dimension and value are lists
+        if type(dimension[0]) is int:
+            dimension[0] = [dimension[0]]
+            dimension[1] = [dimension[1]]
+        indx = numpy.arange(len(self.mixing_matrix[0])).reshape(self.dimensions.shape)
+        # For every axis, get corresponding elements
+        for i, j in sorted(zip(dimension[0], dimension[1])):
+            # Flip axis to be consistent with the data shape
+            axis = self.dimensions.ndim - i - 1
+            # dim_values = numpy.array(list(self.dimensions.values())[dim])
+            indx = indx.take(indices=j, axis=axis)
+        self._plot_rocking_curves.clear()
+        axis = 0
+        for i in range(self.dimensions.ndim):
+            if i not in dimension[0]:
+                axis = i
+        text = self.dimensions.get(axis).name
+        self._plot_rocking_curves.getPlot().setGraphXLabel(text)
+        if len(indx.shape) == 1:
+            for i, row in enumerate(self.mixing_matrix):
+                W = row[indx]
+                self._plot_rocking_curves.addCurve(self.values[axis][indx], W, legend=str(i))
 
     def clearPlots(self):
-        self._plot_rocking_curves.getPlot().clear()
+        self._plot_rocking_curves.clear()
 
 
 class PlotRockingCurves(qt.QMainWindow):
@@ -309,11 +313,11 @@ class PlotRockingCurves(qt.QMainWindow):
     to show only the rocking curves of a certain moving dimension.
     """
 
-    sigFrameChanged = qt.Signal(int)
-    sigMotorChanged = qt.Signal(int)
+    sigStateDisabled = qt.Signal()
+    sigMotorChanged = qt.Signal(list, list)
     sigActiveCurveChanged = qt.Signal(object, object)
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, dimensions=None):
         qt.QMainWindow.__init__(self, parent)
 
         self._plot = Plot1D()
@@ -321,66 +325,24 @@ class PlotRockingCurves(qt.QMainWindow):
         self._plot.setGraphXLabel("Image id")
         self._plot.sigActiveCurveChanged.connect(self.sigActiveCurveChanged)
 
+        self._chooseDimensionDock = ChooseDimensionDock(self, vertical=False)
+        self._chooseDimensionDock.hide()
         self._motors = qt.QWidget()
-        self._motors.setLayout(self._motorsLayout())
         self._motors.hide()
         layout = qt.QVBoxLayout()
         layout.addWidget(self._plot)
-        layout.addWidget(self._motors)
 
         centralWidget = qt.QWidget()
         centralWidget.setLayout(layout)
         self.setCentralWidget(centralWidget)
+        self.addDockWidget(qt.Qt.BottomDockWidgetArea, self._chooseDimensionDock)
 
         self.addCurve = self._plot.addCurve
-
-    def _motorsLayout(self):
-        # If dataset data is reshaped
-        # Creates widget to move through the motors dimensions
-        layout = qt.QHBoxLayout()
-        cb_label = qt.QLabel("Static motor:")
-        self._cb = qt.QComboBox()
-        self._cb.setCurrentIndex(0)
-        # Default: all of the rocking curves are shown
-        self._cb.addItem("None")
-        browser_label = qt.QLabel("Motor index:")
-        self._browser = HorizontalSliderWithBrowser()
-        self._browser.setRange(0, 0)
-        # Browser only activated with choosen dimension
-        self._browser.setEnabled(False)
-
-        # Connect browser
-        self._browser.valueChanged[int].connect(self.sigFrameChanged)
-        # Connect combobox
-        self._cb.currentIndexChanged.connect(self.sigMotorChanged)
-
-        layout.addWidget(cb_label)
-        layout.addWidget(self._cb)
-        layout.addWidget(browser_label)
-        layout.addWidget(self._browser)
-
-        return layout
-
-    def getPlot(self):
-        return self._plot
-
-    def getBrowser(self):
-        return self._browser
-
-    def getDimensionsCB(self):
-        return self._cb
+        self._chooseDimensionDock.widget.filterChanged.connect(self.sigMotorChanged)
+        self._chooseDimensionDock.widget.stateDisabled.connect(self.sigStateDisabled)
 
     def clear(self):
         self._plot.clear()
 
-    def showMotors(self, show=True):
-        self._motors.setVisible(show)
-
-    def addDimension(self, dimension):
-        self._cb.addItem(dimension)
-
-    def resetDimensions(self):
-        oldState = self._cb.blockSignals(True)
-        self._cb.clear()
-        self._cb.addItem("None")
-        self._cb.blockSignals(oldState)
+    def getPlot(self):
+        return self._plot
