@@ -40,7 +40,8 @@ from silx.math.medianfilter import medfilt2d
 from silx.utils.enum import Enum as _Enum
 from silx.io.dictdump import dicttonx
 
-import darfix
+from darfix.io.utils import create_nxdata_dict
+from darfix.gui.utils import ChooseDimensionWidget
 from .operationThread import OperationThread
 
 _logger = logging.getLogger(__file__)
@@ -85,13 +86,23 @@ class GrainPlotWidget(qt.QMainWindow):
         self._levelsLE.setToolTip("Number of levels to use when finding the contours")
         self._levelsLE.setValidator(qt.QIntValidator())
         self._computeContoursB = qt.QPushButton("Compute")
+        self._motorValuesCheckbox = qt.QCheckBox("Use motor values")
+        self._motorValuesCheckbox.setChecked(True)
+        self._motorValuesCheckbox.stateChanged.connect(self._checkboxStateChanged)
         self._centerDataCheckbox = qt.QCheckBox("Center angle values")
+        self._centerDataCheckbox.setEnabled(False)
         self._centerDataCheckbox.stateChanged.connect(self._checkboxStateChanged)
+        self._chooseDimensionWidget = ChooseDimensionWidget(self, vertical=False, values=False, _filter=False)
+        self._chooseDimensionWidget.filterChanged.connect(self._updateMotorAxis)
+        self._thirdMotorCB = qt.QComboBox(self)
+        self._thirdMotorCB.currentIndexChanged.connect(self._updateThirdMotor)
         levelsLayout.addWidget(levelsLabel, 0, 0, 1, 1)
         levelsLayout.addWidget(self._levelsLE, 0, 1, 1, 1)
-        levelsLayout.addWidget(self._centerDataCheckbox, 0, 2, 1, 1)
-        levelsLayout.addWidget(self._computeContoursB, 1, 2, 1, 1)
-        levelsLayout.addWidget(self._contoursPlot, 2, 0, 1, 3)
+        levelsLayout.addWidget(self._motorValuesCheckbox, 0, 2, 1, 1)
+        levelsLayout.addWidget(self._centerDataCheckbox, 0, 3, 1, 1)
+        levelsLayout.addWidget(self._thirdMotorCB, 1, 2, 1, 1)
+        levelsLayout.addWidget(self._computeContoursB, 1, 3, 1, 1)
+        levelsLayout.addWidget(self._contoursPlot, 2, 0, 1, 4)
         self._levelsWidget.setLayout(levelsLayout)
         self._levelsWidget.hide()
         self._mosaicityPlot = Plot2D(parent=self)
@@ -99,15 +110,17 @@ class GrainPlotWidget(qt.QMainWindow):
         self._exportButton.setEnabled(False)
         self._exportButton.clicked.connect(self.exportMaps)
         layout.addWidget(self._methodCB)
+        layout.addWidget(self._chooseDimensionWidget)
         layout.addWidget(self._levelsWidget)
         layout.addWidget(self._plotWidget)
         layout.addWidget(self._mosaicityPlot)
         layout.addWidget(self._exportButton)
         self._plotWidget.hide()
+        self._thirdMotorCB.hide()
+        self._chooseDimensionWidget.hide()
         self._mosaicityPlot.hide()
         self._mosaicityPlot.getColorBarWidget().hide()
         widget.setLayout(layout)
-        widget.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Minimum)
         self.setCentralWidget(widget)
 
     def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
@@ -121,27 +134,40 @@ class GrainPlotWidget(qt.QMainWindow):
         self.indices = indices
         self.bg_indices = bg_indices
         self.bg_dataset = bg_dataset
+        self._mosaicity = None
+        self._moments = None
+        self.dimensions = [0, 1]
+        state = self._methodCB.blockSignals(True)
         for i in range(len(Method)):
             self._methodCB.model().item(i).setEnabled(False)
-        scale = 100
+        self._methodCB.blockSignals(state)
         if self._dataset.transformation:
             transformation = self._dataset.transformation.transformation
-            px = transformation[0][0][0]
-            py = transformation[1][0][0]
-            xscale = (transformation[0][-1][-1] - px) / transformation[0].shape[1]
-            yscale = (transformation[1][-1][-1] - py) / transformation[1].shape[0]
+            eta = numpy.degrees(transformation[0])
+            twotheta = numpy.degrees(transformation[1])
+            px = eta[0][0]
+            py = twotheta[0][0]
+            xscale = (eta[-1][-1] - px) / eta.shape[1]
+            yscale = (twotheta[-1][-1] - py) / twotheta.shape[0]
             self.origin = (px, py)
             self.scale = (xscale, yscale)
-        if self._dataset.dims.ndim == 2:
+        if self._dataset.dims.ndim > 1 and self._dataset.dims.ndim < 4:
             self._curves = {}
-            self.ori_dist, self.hsv_key = self._dataset.compute_mosaicity_colorkey()
-            xdim = self._dataset.dims.get(1)
-            ydim = self._dataset.dims.get(0)
-            xscale = (xdim.unique_values[-1] - xdim.unique_values[0]) / (xdim.size - 1)
-            yscale = (ydim.unique_values[-1] - ydim.unique_values[0]) / (ydim.size - 1)
-
-            self._contoursPlot.addImage(hsv_to_rgb(self.hsv_key), xlabel=xdim.name,
-                                        ylabel=ydim.name, scale=(xscale / scale, yscale / scale))
+            if self._dataset.dims.ndim == 3:
+                # Update chooseDimensionWidget and thirdMotorCB
+                state = self._chooseDimensionWidget.blockSignals(True)
+                self._chooseDimensionWidget.setDimensions(self._dataset.dims)
+                self._chooseDimensionWidget._updateState(True)
+                self._chooseDimensionWidget.blockSignals(state)
+                self._thirdMotorCB.clear()
+                third_motor = self._dataset.dims.get(
+                    int(numpy.setdiff1d(range(3), self._chooseDimensionWidget.dimension)[0])
+                )
+                state = self._thirdMotorCB.blockSignals(True)
+                self._thirdMotorCB.addItems(numpy.array(third_motor.unique_values, dtype=str))
+                self._thirdMotorCB.blockSignals(state)
+            self.ori_dist, self.hsv_key = self._dataset.compute_mosaicity_colorkey(third_motor=0)
+            self._checkboxStateChanged()
             self._contoursPlot.getColorBarWidget().hide()
             self._curvesColormap = Colormap(name='temperature',
                                             vmin=numpy.min(self.ori_dist),
@@ -156,10 +182,12 @@ class GrainPlotWidget(qt.QMainWindow):
         for i in reversed(range(self._plotWidget.layout().count())):
             self._plotWidget.layout().itemAt(i).widget().setParent(None)
 
+        self._contoursPlot.setGraphTitle(self._dataset.title + "\n" + Method.ORI_DIST.value)
+        self._mosaicityPlot.setGraphTitle(self._dataset.title + "\n" + Method.MOSAICITY.value)
         self._plots = []
         for axis, dim in self._dataset.dims:
             self._plots += [Plot2D(parent=self)]
-            self._plots[-1].setGraphTitle(dim.name)
+            self._plots[-1].setGraphTitle(self._dataset.title + "\n" + dim.name)
             self._plots[-1].setDefaultColormap(Colormap(name='viridis'))
             self._plotWidget.layout().addWidget(self._plots[-1])
 
@@ -170,37 +198,54 @@ class GrainPlotWidget(qt.QMainWindow):
         self._thread.finished.disconnect(self._updateData)
         if self._thread.data is not None:
             self._moments = self._thread.data
+            if self._dataset.transformation and self._dataset.transformation.rotate:
+                for axis in self._moments:
+                    self._moments[axis] = [numpy.rot90(moment, 3) for moment in self._moments[axis]]
             self._updatePlot(self._methodCB.currentText())
             rg = len(Method) if self._dataset.dims.ndim > 1 else 4
             for i in range(rg):
                 self._methodCB.model().item(i).setEnabled(True)
+            self.mosaicity = self.compute_mosaicity()
             self._methodCB.setCurrentIndex(0)
             self._exportButton.setEnabled(True)
         else:
             print("\nComputation aborted")
+            msg = qt.QMessageBox()
+            msg.setIcon(qt.QMessageBox.Critical)
+            msg.setText("Grain plot not working without dimensions")
+            msg.setWindowTitle("Don't eat yellow snow!")
+            msg.setStandardButtons(qt.QMessageBox.Ok)
+            msg.exec_()
 
     def _updateDataset(self, widget, dataset):
         self._parent._updateDataset(widget, dataset)
         self._dataset = dataset
 
-    def _checkboxStateChanged(self, state):
+    def _checkboxStateChanged(self, state=None):
         """
         Update widgets linked to the checkbox state
         """
-        scale = 100
-        xdim = self._dataset.dims.get(1)
-        ydim = self._dataset.dims.get(0)
-        xsize = xdim.size - 1
-        ysize = ydim.size - 1
-        xscale = (xdim.unique_values[-1] - xdim.unique_values[0]) / xsize
-        yscale = (ydim.unique_values[-1] - ydim.unique_values[0]) / ysize
-        origin = (- xscale * xsize / 2, - yscale * ysize / 2) if state else (0., 0.)
-        self._contoursPlot.addImage(hsv_to_rgb(self.hsv_key), xlabel=xdim.name, ylabel=ydim.name,
-                                    origin=origin, scale=(xscale / scale, yscale / scale))
-
+        self._centerDataCheckbox.setEnabled(not self._motorValuesCheckbox.isChecked())
         self._contoursPlot.remove(kind='curve')
+        self._contoursPlot.resetZoom()
+        scale = 100
+        xdim = self._dataset.dims.get(self.dimensions[1])
+        ydim = self._dataset.dims.get(self.dimensions[0])
+        xscale = xdim.range[2]
+        yscale = ydim.range[2]
+        if self._motorValuesCheckbox.isChecked():
+            self.ori_dist_origin = (xdim.range[0], ydim.range[0])
+        elif self._centerDataCheckbox.isChecked():
+            self.ori_dist_origin = (- xscale * int(xdim.size / 2), - yscale * int(ydim.size / 2))
+        else:
+            self.ori_dist_origin = (0, 0)
+        self._contoursPlot.addImage(hsv_to_rgb(self.hsv_key), xlabel=xdim.name, ylabel=ydim.name,
+                                    origin=self.ori_dist_origin, scale=(xscale / scale, yscale / scale))
 
     def _computeContours(self):
+        """
+        Compute contours map based on orientation distribution.
+        """
         self._contoursPlot.remove(kind='curve')
 
         if self.ori_dist is not None:
@@ -211,8 +256,8 @@ class GrainPlotWidget(qt.QMainWindow):
                 levels.append(i)
 
             colors = self._curvesColormap.applyToData(levels)
-            xdim = self._dataset.dims.get(1)
-            ydim = self._dataset.dims.get(0)
+            xdim = self._dataset.dims.get(self.dimensions[1])
+            ydim = self._dataset.dims.get(self.dimensions[0])
             self._curves = {}
             for ipolygon, polygon in enumerate(polygons):
                 # iso contours
@@ -222,15 +267,10 @@ class GrainPlotWidget(qt.QMainWindow):
                     # isClosed = numpy.allclose(contour[0], contour[-1])
                     x = contour[:, 1]
                     y = contour[:, 0]
-                    rescale_x = (xdim.unique_values[-1] - xdim.unique_values[0]) / (xdim.size - 1)
-                    rescale_y = (ydim.unique_values[-1] - ydim.unique_values[0]) / (ydim.size - 1)
-                    x *= rescale_x
-                    y *= rescale_y
-                    if self._centerDataCheckbox.isChecked():
-                        xcenter = (xdim.unique_values[-1] - xdim.unique_values[0]) / 2
-                        x -= xcenter
-                        ycenter = (ydim.unique_values[-1] - ydim.unique_values[0]) / 2
-                        y -= ycenter
+                    xscale = xdim.range[2]
+                    yscale = ydim.range[2]
+                    x = self.ori_dist_origin[0] + x * xscale + xscale / 2
+                    y = self.ori_dist_origin[1] + y * yscale + yscale / 2
                     legend = "poly{}.{}".format(icontour, ipolygon)
                     self._curves[legend] = {
                         "points": (x.copy(), y.copy()),
@@ -240,40 +280,59 @@ class GrainPlotWidget(qt.QMainWindow):
                                                 legend=legend, resetzoom=False,
                                                 color=colors[ipolygon])
 
-    def _computeMosaicity(self):
+    def compute_mosaicity(self):
+        """
+        Compute mosaicity map depending on the selected dimensions, if any.
+        """
+        if self._moments is not None and self._dataset.dims.ndim > 1:
+            com0 = self._moments[self.dimensions[0]][0]
+            no_nans_indices = ~numpy.isnan(com0)
+            min_com0 = numpy.min(com0[no_nans_indices] if len(com0[no_nans_indices]) > 0 else com0)
+            com0[numpy.isnan(com0)] = min_com0
+            norms0 = (com0 - min_com0) / numpy.ptp(com0)
+            com1 = self._moments[self.dimensions[1]][0]
+            no_nans_indices = ~numpy.isnan(com1)
+            min_com1 = numpy.min(com1[no_nans_indices] if len(com1[no_nans_indices]) > 0 else com1)
+            com1[numpy.isnan(com1)] = min_com1
+            norms1 = (com1 - min_com1) / numpy.ptp(com1)
 
-        norms0 = (self._moments[0][0] - numpy.min(self._moments[0][0])) / numpy.ptp(self._moments[0][0])
-        norms1 = (self._moments[1][0] - numpy.min(self._moments[1][0])) / numpy.ptp(self._moments[1][0])
-
-        mosaicity = numpy.stack((norms0, norms1, numpy.ones(self._moments[0].shape[1:])), axis=2)
-        return mosaicity
+            mosaicity = hsv_to_rgb(numpy.stack((norms0, norms1, numpy.ones(self._moments[0].shape[1:])), axis=2))
+            if self._dataset.transformation and self._dataset.transformation.rotate:
+                mosaicity = numpy.rot90(mosaicity, 3)
+            return mosaicity
 
     def _updatePlot(self, method):
+        """
+        Update shown plots in the widget
+        """
         method = Method(method)
         self._levelsWidget.hide()
         self._mosaicityPlot.hide()
+        self._thirdMotorCB.hide()
+        self._chooseDimensionWidget.hide()
         if method == Method.ORI_DIST:
             self._levelsWidget.show()
             self._plotWidget.hide()
+            if self._dataset.dims.ndim == 3:
+                self._chooseDimensionWidget.show()
+                self._thirdMotorCB.show()
         elif method == Method.FWHM:
             self._plotWidget.show()
             if self._dataset.transformation is not None:
                 label = self._dataset.transformation.label
                 for i, plot in enumerate(self._plots):
-                    plot.addImage(darfix.config.FWHM_VAL * (numpy.rot90(self._moments[i][1], 3)
-                                  if self._dataset.transformation.rotate else self._moments[i][1]),
+                    plot.addImage(self._moments[i][1],
                                   origin=self.origin, scale=self.scale,
                                   xlabel=label, ylabel=label)
             else:
                 for i, plot in enumerate(self._plots):
-                    plot.addImage(darfix.config.FWHM_VAL * self._moments[i][1])
+                    plot.addImage(self._moments[i][1])
         elif method == Method.COM:
             self._plotWidget.show()
             if self._dataset.transformation is not None:
                 label = self._dataset.transformation.label
                 for i, plot in enumerate(self._plots):
-                    plot.addImage(numpy.rot90(self._moments[i][0], 3) if self._dataset.transformation.rotate
-                                  else self._moments[i][0], origin=self.origin, scale=self.scale,
+                    plot.addImage(self._moments[i][0], origin=self.origin, scale=self.scale,
                                   xlabel=label, ylabel=label)
             else:
                 for i, plot in enumerate(self._plots):
@@ -283,8 +342,7 @@ class GrainPlotWidget(qt.QMainWindow):
             if self._dataset.transformation is not None:
                 label = self._dataset.transformation.label
                 for i, plot in enumerate(self._plots):
-                    plot.addImage(numpy.rot90(self._moments[i][2], 3) if self._dataset.transformation.rotate
-                                  else self._moments[i][2], origin=self.origin, scale=self.scale,
+                    plot.addImage(self._moments[i][2], origin=self.origin, scale=self.scale,
                                   xlabel=label, ylabel=label)
             else:
                 for i, plot in enumerate(self._plots):
@@ -294,8 +352,7 @@ class GrainPlotWidget(qt.QMainWindow):
             if self._dataset.transformation is not None:
                 label = self._dataset.transformation.label
                 for i, plot in enumerate(self._plots):
-                    plot.addImage(numpy.rot90(self._moments[i][3], 3) if self._dataset.transformation.rotate
-                                  else self._moments[i][3], origin=self.origin, scale=self.scale,
+                    plot.addImage(self._moments[i][3], origin=self.origin, scale=self.scale,
                                   xlabel=label, ylabel=label)
             else:
                 for i, plot in enumerate(self._plots):
@@ -305,16 +362,58 @@ class GrainPlotWidget(qt.QMainWindow):
                 self._plotWidget.hide()
                 if self._dataset.transformation:
                     label = self._dataset.transformation.label
-                    image = hsv_to_rgb(self._computeMosaicity())
-                    self._mosaicityPlot.addImage(numpy.rot90(image, 3) if self._dataset.transformation.rotate
-                                                 else image, origin=self.origin, scale=self.scale,
-                                                 xlabel=label, ylabel=label)
+                    self._mosaicityPlot.addImage(self.mosaicity, origin=self.origin,
+                                                 scale=self.scale, xlabel=label, ylabel=label)
                 else:
-                    self._mosaicityPlot.addImage(hsv_to_rgb(self._computeMosaicity()))
+                    self._mosaicityPlot.addImage(self.mosaicity)
 
                 self._mosaicityPlot.show()
-            except Exception:
-                _logger.error("Couldn't compute mosaicity")
+                if self._dataset.dims.ndim == 3:
+                    self._chooseDimensionWidget.show()
+            except Exception as e:
+                _logger.error("Couldn't compute mosaicity: ", e)
+        else:
+            _logger.warning("Unexisting map method")
+
+    def _updateMotorAxis(self):
+        """
+        Update dimensions used for orientation distribution and mosaicity maps.
+        This is only used on datasets with more than two dimensions.
+        """
+
+        self.dimensions = self._chooseDimensionWidget.dimension
+        self.ori_dist, self.hsv_key = self._dataset.compute_mosaicity_colorkey(
+            dimensions=self.dimensions, third_motor=0
+        )
+        self.mosaicity = self.compute_mosaicity()
+
+        self._thirdMotorCB.clear()
+        third_motor = self._dataset.dims.get(
+            int(numpy.setdiff1d(range(3), self.dimensions)[0])
+        )
+
+        self._curvesColormap = Colormap(name='temperature',
+                                        vmin=numpy.min(self.ori_dist),
+                                        vmax=numpy.max(self.ori_dist))
+        state = self._thirdMotorCB.blockSignals(True)
+        self._thirdMotorCB.addItems(numpy.array(third_motor.unique_values, dtype=str))
+        self._thirdMotorCB.blockSignals(state)
+        self._checkboxStateChanged()
+        self._updatePlot(self._methodCB.currentText())
+
+    def _updateThirdMotor(self, index=-1):
+        """
+        Update orientation distribution according to the third motor chosen
+        """
+        if index > -1:
+            self.ori_dist, self.hsv_key = self._dataset.compute_mosaicity_colorkey(
+                dimensions=self._chooseDimensionWidget.dimension, third_motor=index
+            )
+
+            self._curvesColormap = Colormap(name='temperature',
+                                            vmin=numpy.min(self.ori_dist),
+                                            vmax=numpy.max(self.ori_dist))
+            self._updatePlot(self._methodCB.currentText())
 
     def _opticolor(self, img, minc, maxc):
         img = img.copy()
@@ -331,22 +430,29 @@ class GrainPlotWidget(qt.QMainWindow):
         """
         Creates dictionay with maps information and exports it to a nexus file
         """
+        if self._dataset.transformation:
+            shape = self._dataset.get_data(0).shape
+            axes = [[self.origin[1] + i * self.scale[1] for i in range(shape[0])],
+                    [self.origin[0] + i * self.scale[0] for i in range(shape[1])]]
+            axes_names = ["y", "x"]
+            axes_long_names = [self._dataset.transformation.label,
+                               self._dataset.transformation.label]
+        else:
+            axes = None
+            axes_names = None
+            axes_long_names = None
+
         if self._dataset and self._dataset.dims.ndim > 1:
             nx = {
                 "entry": {
-                    "MOSAICITY": {
-                        ">" + Method.MOSAICITY.name: "../maps/" + Method.MOSAICITY.name,
-                        "@signal": Method.MOSAICITY.name,
-                        "@NX_class": "NXdata"
-                    },
-                    "maps": {
-                        Method.MOSAICITY.name: hsv_to_rgb(self._computeMosaicity()),
-                        Method.MOSAICITY.name + "@interpretation": "rgba-image",
-                        "@NX_class": "NXcollection"
-                    },
-                    Method.ORI_DIST.name: {
+                    Method.MOSAICITY.value: create_nxdata_dict(
+                        self.mosaicity, Method.MOSAICITY.value, axes, axes_names,
+                        axes_long_names, rgba=True
+                    ),
+                    Method.ORI_DIST.value: {
                         "key": {
                             "image": hsv_to_rgb(self.hsv_key),
+                            "origin": self._contoursPlot.getImage().getOrigin(),
                             "scale": self._contoursPlot.getImage().getScale(),
                             "xlabel": self._contoursPlot.getImage().getXLabel(),
                             "ylabel": self._contoursPlot.getImage().getYLabel(),
@@ -355,41 +461,35 @@ class GrainPlotWidget(qt.QMainWindow):
                         "curves": self._curves
                     },
                     "@NX_class": "NXentry",
-                    "@default": "data",
+                    "@default": Method.MOSAICITY.value,
                 },
                 "@NX_class": "NXroot",
                 "@default": "entry"
             }
 
             for axis, dim in self._dataset.dims:
-                nx["entry"]["maps"][dim.name] = {
-                    Method.COM.name: self._moments[axis][0],
-                    Method.FWHM.name: self._moments[axis][1],
-                    Method.SKEWNESS.name: self._moments[axis][2],
-                    Method.KURTOSIS.name: self._moments[axis][3]
+                nx["entry"][dim.name] = {
+                    "@NX_class": "NXcollection"
                 }
+                for i in range(4):
+                    nx["entry"][dim.name][Method.values()[i]] = create_nxdata_dict(
+                        self._moments[axis][i], Method.values()[i], axes, axes_names,
+                        axes_long_names)
         else:
             nx = {
                 "entry": {
-                    "data": {
-                        ">" + Method.COM.name: "../maps/" + Method.COM.name,
-                        "@signal": Method.COM.name,
-                        "@NX_class": "NXdata"
-                    },
-                    "maps": {
-                        Method.COM.name: self._moments[0][0],
-                        Method.FWHM.name: self._moments[0][1],
-                        Method.SKEWNESS.name: self._moments[0][2],
-                        Method.KURTOSIS.name: self._moments[0][3],
-                        "@NX_class": "NXcollection"
-                    },
-                    "@NX_class": "NXentry",
-                    "@default": "data",
+                    "@NX_class": "NXentry"
                 },
                 "@NX_class": "NXroot",
                 "@default": "entry"
             }
 
+            for i in range(4):
+                nx["entry"][Method.values()[i]] = create_nxdata_dict(
+                    self._moments[0][i], Method.values()[i], axes, axes_names,
+                    axes_long_names)
+            nx["entry"]["@default"] = Method.COM.value
+
         fileDialog = qt.QFileDialog()
 
         fileDialog.setFileMode(fileDialog.AnyFile)
