@@ -36,6 +36,7 @@ from silx.gui.colors import Colormap
 from silx.gui.plot import StackView
 from silx.gui.plot.items import Scatter
 import darfix
+from darfix.core.dimension import AcquisitionDims
 from darfix.core.componentsMatching import ComponentsMatching, Method
 from darfix.gui.datasetSelectionWidget import FilenameSelectionWidget
 from darfix.gui.displayComponentsWidget import MixingPlotsWidget
@@ -150,13 +151,15 @@ class LinkComponentsWidget(qt.QMainWindow):
                 msg.exec_()
             return
 
-        self.dimensions1, self.components1, self.W1, shape = read_components(filename)
+        dims, self.components1, self.W1, values = read_components(filename)
+        self.dimensions1 = AcquisitionDims()
+        self.dimensions1.from_dict(dims)
 
         self._linked_sv.hide()
         self._sv1.setStack(self.components1)
         self._displayComponents[0] = True
         self._sv1.show()
-        self._mixingPlots1.updatePlots(self.dimensions1, self.W1.T, shape)
+        self._mixingPlots1.updatePlots(self.dimensions1, self.W1.T, values)
         self._mixingPlots1.show()
 
         if all(self._displayComponents):
@@ -174,13 +177,15 @@ class LinkComponentsWidget(qt.QMainWindow):
         if filename == '':
             return
 
-        self.dimensions2, self.components2, self.W2, shape = read_components(filename)
+        dims, self.components2, self.W2, values = read_components(filename)
+        self.dimensions2 = AcquisitionDims()
+        self.dimensions2.from_dict(dims)
 
         self._linked_sv.hide()
         self._sv2.setStack(self.components2)
         self._displayComponents[1] = True
         self._sv2.show()
-        self._mixingPlots2.updatePlots(self.dimensions2, self.W2.T, shape)
+        self._mixingPlots2.updatePlots(self.dimensions2, self.W2.T, values)
         self._mixingPlots2.show()
 
         if all(self._displayComponents):
