@@ -41,6 +41,11 @@ class Value(_Enum):
     PIXEL_10X = 0.75
 
 
+class Orientation(_Enum):
+    VERTICAL = "Vertical"
+    HORIZONTAL = "Horizontal"
+
+
 class MagnificationWidget(qt.QMainWindow):
     """
     Widget to apply magnification transformation to the data axes.
@@ -61,7 +66,7 @@ class MagnificationWidget(qt.QMainWindow):
         self._centerAxesCheckbox = qt.QCheckBox("Center axes")
         self._centerAxesCheckbox.setChecked(True)
         self._orientationCB = qt.QComboBox()
-        self._orientationCB.addItems(["Vertical", "Horizontal"])
+        self._orientationCB.addItems(Orientation.values())
         topographyAxis = qt.QLabel("Topography axis: ")
         self._manualLE = qt.QLineEdit(parent=self)
         self._manualLE.setEnabled(False)
@@ -123,6 +128,19 @@ class MagnificationWidget(qt.QMainWindow):
     def magnification(self):
         return self._magnification
 
+    @property
+    def orientation(self):
+        if self._topographyCheckbox.isChecked():
+            return self._orientationCB.currentIndex()
+        else:
+            return -1
+
+    @orientation.setter
+    def orientation(self, orientation):
+        if orientation != -1:
+            self._topographyCheckbox.setChecked(True)
+            self._orientationCB.setCurrentIndex(orientation)
+
     @magnification.setter
     def magnification(self, magnification):
         self._magnification = magnification
