@@ -26,14 +26,14 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "14/10/2019"
+__date__ = "08/04/2020"
 
 
 import numpy
 
 from silx.gui import qt
 
-from darfix.core.dataset import _METADATA_TYPES
+from darfix.core.dimension import _METADATA_TYPES
 
 
 class MetadataWidget(qt.QMainWindow):
@@ -66,8 +66,9 @@ class MetadataWidget(qt.QMainWindow):
         self.setCentralWidget(mainWidget)
         self._metadataTypeCB.currentTextChanged.connect(self._updateView)
 
-    def setDataset(self, dataset):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         self._dataset = dataset
+        self._parent = parent
         self._updateView()
 
     def clearTable(self):
@@ -85,29 +86,33 @@ class MetadataWidget(qt.QMainWindow):
 
         self._table.clear()
         # v_header = [str(i) for i in range(self.__experiment.nslices)]
-        metadata = self._dataset.metadata
+        metadata = self._dataset.get_data().metadata
         self._table.setRowCount(len(metadata))
         # self._table.setVerticalHeaderLabels(v_header)
 
         columnCount = None
-        for row, metadata_frame in enumerate(self._dataset.metadata):
-            keys = metadata_frame.get_keys(kind=metadata_type)
-            if not row:
-                if columnCount is None:
-                    self._table.setColumnCount(len(keys))
-                    self._table.setHorizontalHeaderLabels(keys)
-                elif columnCount != len(metadata_frame):
-                    raise ValueError('Metadata keys are incoherent')
+        if not self._dataset._isH5:
+            for row, metadata_frame in enumerate(metadata):
+                keys = metadata_frame.get_keys(kind=metadata_type)
+                if not row:
+                    if columnCount is None:
+                        self._table.setColumnCount(len(keys))
+                        self._table.setHorizontalHeaderLabels(keys)
+                    elif columnCount != len(metadata_frame):
+                        raise ValueError('Metadata keys are incoherent')
 
-            for column, key in enumerate(keys):
-                _item = qt.QTableWidgetItem()
-                txt = metadata_frame.get_value(kind=metadata_type, name=key)
-                if type(txt) is numpy.ndarray and txt.size == 1:
-                    txt = txt[0]
-                if hasattr(txt, 'decode'):
-                    txt = txt.decode('utf-8')
-                else:
-                    txt = str(txt)
-                _item.setText(txt)
-                _item.setFlags(qt.Qt.ItemIsEnabled | qt.Qt.ItemIsSelectable)
-                self._table.setItem(row, column, _item)
+                for column, key in enumerate(keys):
+                    _item = qt.QTableWidgetItem()
+                    try:
+                        txt = metadata_frame.get_value(kind=metadata_type, name=key)
+                    except KeyError:
+                        txt = "0"
+                    if type(txt) is numpy.ndarray and txt.size == 1:
+                        txt = txt[0]
+                    if hasattr(txt, 'decode'):
+                        txt = txt.decode('utf-8')
+                    else:
+                        txt = str(txt)
+                    _item.setText(txt)
+                    _item.setFlags(qt.Qt.ItemIsEnabled | qt.Qt.ItemIsSelectable)
+                    self._table.setItem(row, column, _item)
