@@ -33,6 +33,7 @@ import logging
 from silx.gui import qt
 from silx.gui.colors import Colormap
 from silx.gui.plot.StackView import StackViewMainWindow
+from silx.utils.enum import Enum as _Enum
 
 import darfix
 from darfix.core.dataset import Operation
@@ -43,6 +44,12 @@ from .operationThread import OperationThread
 _logger = logging.getLogger(__file__)
 
 
+class DATA_TYPES(_Enum):
+    DATA = "Data"
+    UNUSED_DATA = "Unused data (after partition)"
+    DARK_DATA = "Dark data"
+
+
 class _ParametersDock(qt.QDockWidget):
 
     def __init__(self, parent=None):
@@ -147,6 +154,19 @@ class _ParametersDock(qt.QDockWidget):
         self.computeTP.hide()
         self.abortTP.hide()
 
+        # Mask removal
+        mrLabel = qt.QLabel("Mask Removal")
+        mrLabel.setFont(font)
+        self.layout.addWidget(mrLabel, 0, 6, 1, 2)
+        maskLabel = qt.QLabel("Use mask from toolbox.\n Set values off mask to 0.")
+        self.computeMR = qt.QPushButton("Compute")
+        self.abortMR = qt.QPushButton("Abort")
+        self.layout.addWidget(maskLabel, 1, 6, 1, 2)
+        self.layout.addWidget(self.computeMR, 4, 7, 1, 1)
+        self.layout.addWidget(self.abortMR, 4, 6, 1, 1)
+        self.computeMR.hide()
+        self.abortMR.hide()
+
         self.layout.setHorizontalSpacing(10)
         widget.setLayout(self.layout)
         self.setWidget(widget)
@@ -183,11 +203,11 @@ class NoiseRemovalDialog(qt.QDialog):
         resetB.clicked.connect(self.mainWindow.resetStack)
         self.mainWindow.computingSignal.connect(self._toggleButton)
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
 
         if dataset is not None:
             self._buttons.setEnabled(True)
-            self.mainWindow.setDataset(dataset, indices, bg_indices, bg_dataset)
+            self.mainWindow.setDataset(parent, dataset, indices, bg_indices, bg_dataset)
 
     def _toggleButton(self, state):
         self._buttons.button(qt.QDialogButtonBox.Ok).setEnabled(not state)
@@ -226,6 +246,7 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._step = self._parametersDock.step.text()
         self._chunks = [int(self._parametersDock.verticalChunkSize.text()),
                         int(self._parametersDock.horizontalChunkSize.text())]
+        self._mask = None
         self.setCentralWidget(self._sv)
         self.addDockWidget(qt.Qt.BottomDockWidgetArea, self._parametersDock)
 
@@ -233,22 +254,27 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._parametersDock.computeBS.clicked.connect(self.__computeBS)
         self._parametersDock.computeHP.clicked.connect(self.__computeHP)
         self._parametersDock.computeTP.clicked.connect(self.__computeTP)
+        self._parametersDock.computeMR.clicked.connect(self.__computeMR)
         self._parametersDock.abortBS.clicked.connect(self.__abortBS)
         self._parametersDock.abortHP.clicked.connect(self.__abortHP)
         self._parametersDock.abortTP.clicked.connect(self.__abortTP)
+        self._parametersDock.abortMR.clicked.connect(self.__abortMR)
         self._parametersDock.bsMethodsCB.currentTextChanged.connect(self._toggleMethod)
         self._parametersDock.bsBackgroundCB.currentIndexChanged.connect(self._toggleInDiskWidget)
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
         Dataset setter. Saves the dataset and updates the stack with the dataset
         data
 
         :param Dataset dataset: dataset
         """
-        self.dataset = dataset
+        self._parent = parent
+        self._dataset = dataset
         self._update_dataset = dataset
         self.indices = indices
+        if self._dataset.title != "":
+            self._sv.setTitleCallback(lambda idx: self._dataset.title)
         self.setStack()
         self.bg_indices = bg_indices
         self.bg_dataset = bg_dataset
@@ -256,16 +282,17 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._parametersDock.computeBS.show()
         self._parametersDock.computeHP.show()
         self._parametersDock.computeTP.show()
+        self._parametersDock.computeMR.show()
 
         """
         Sets the available background for the user to choose.
         """
         self._parametersDock.bsBackgroundCB.clear()
         if bg_dataset is not None:
-            self._parametersDock.bsBackgroundCB.addItem("Dark data")
-        if bg_indices is not None:  # TODO: modify
-            self._parametersDock.bsBackgroundCB.addItem("Low intensity data")
-        self._parametersDock.bsBackgroundCB.addItem("Data")
+            self._parametersDock.bsBackgroundCB.addItem(DATA_TYPES.DARK_DATA.value)
+        if bg_indices is not None:
+            self._parametersDock.bsBackgroundCB.addItem(DATA_TYPES.UNUSED_DATA.value)
+        self._parametersDock.bsBackgroundCB.addItem(DATA_TYPES.DATA.value)
 
     def __computeBS(self):
         """
@@ -273,9 +300,9 @@ class NoiseRemovalWidget(qt.QMainWindow):
         subtraction.
         """
         self._background = self._parametersDock.bsBackgroundCB.currentText()
-        if self._background == "Dark data":
+        if self._background == DATA_TYPES.DARK_DATA.value:
             bg = self.bg_dataset
-        elif self._background == "Low intensity data":
+        elif self._background == DATA_TYPES.UNUSED_DATA.value:
             bg = self.bg_indices
         else:
             bg = None
@@ -319,13 +346,28 @@ class NoiseRemovalWidget(qt.QMainWindow):
         """
         self._bottom_threshold = self._parametersDock.bottomLE.text()
         self._thread = OperationThread(self, self._update_dataset.apply_threshold_removal)
-        self._thread.setArgs(int(self._bottom_threshold))
+        self._thread.setArgs(int(self._bottom_threshold), indices=self.indices)
         self._thread.finished.connect(self._updateData)
         if not self._update_dataset.in_memory:
             self._parametersDock.abortTP.show()
         self._thread.start()
         self._setComputingState(True)
 
+    def __computeMR(self):
+        """
+        Function that starts the thread to compute the threshold
+        removal.
+        """
+        self.mask = self._sv.getPlotWidget().getSelectionMask()
+        if self.mask is not None:
+            self._thread = OperationThread(self, self._update_dataset.apply_mask_removal)
+            self._thread.setArgs(self.mask, indices=self.indices)
+            self._thread.finished.connect(self._updateData)
+            if not self._update_dataset.in_memory:
+                self._parametersDock.abortMR.show()
+            self._thread.start()
+            self._setComputingState(True)
+
     def __abortBS(self):
         self._parametersDock.abortBS.setEnabled(False)
         self._update_dataset.stop_operation(Operation.BS)
@@ -338,11 +380,16 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._parametersDock.abortTP.setEnabled(False)
         self._update_dataset.stop_operation(Operation.THRESHOLD)
 
+    def __abortMR(self):
+        self._parametersDock.abortMR.setEnabled(False)
+        self._update_dataset.stop_operation(Operation.MASK)
+
     def _setComputingState(self, state):
 
         self._parametersDock.computeBS.setEnabled(not state)
         self._parametersDock.computeHP.setEnabled(not state)
         self._parametersDock.computeTP.setEnabled(not state)
+        self._parametersDock.computeMR.setEnabled(not state)
 
         self.computingSignal.emit(state)
 
@@ -357,12 +404,17 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._parametersDock.abortBS.setEnabled(True)
         self._parametersDock.abortHP.setEnabled(True)
         self._parametersDock.abortTP.setEnabled(True)
+        self._parametersDock.abortMR.setEnabled(True)
         self._parametersDock.abortBS.hide()
         self._parametersDock.abortHP.hide()
         self._parametersDock.abortTP.hide()
+        self._parametersDock.abortMR.hide()
         self._setComputingState(False)
         if self._thread.data is not None:
+            self.clearStack()
             self._update_dataset = self._thread.data
+            self._thread.func = None
+            self._thread.data = None
             self.setStack(self._update_dataset)
         else:
             print("\nComputation aborted")
@@ -376,9 +428,9 @@ class NoiseRemovalWidget(qt.QMainWindow):
     def _toggleInDiskWidget(self, index):
         if self._parametersDock.bsMethodsCB.currentText() == Method.median.value:
             if self.bg_dataset is None:
-                self._parametersDock.inDiskWidget.hide() if self.dataset.in_memory else \
+                self._parametersDock.inDiskWidget.hide() if self._dataset.in_memory else \
                     self._parametersDock.inDiskWidget.show()
-            elif not (index or self.bg_dataset.in_memory) or (index and not self.dataset.in_memory):
+            elif not (index or self.bg_dataset.in_memory) or (index and not self._dataset.in_memory):
                 self._parametersDock.inDiskWidget.show()
             else:
                 self._parametersDock.inDiskWidget.hide()
@@ -388,8 +440,13 @@ class NoiseRemovalWidget(qt.QMainWindow):
     def getDataset(self):
         return self._update_dataset, self.indices, self.bg_indices, self. bg_dataset
 
+    def _updateDataset(self, widget, dataset):
+        self._dataset = dataset
+        self._update_dataset = dataset
+        self._parent._updateDataset(widget, dataset)
+
     def resetStack(self):
-        self._update_dataset = self.dataset
+        self._update_dataset = self._dataset
         self.setStack()
 
     def clearStack(self):
@@ -406,7 +463,7 @@ class NoiseRemovalWidget(qt.QMainWindow):
         :param Dataset dataset: if not None, data set to the stack will be from the given dataset.
         """
         if dataset is None:
-            dataset = self.dataset
+            dataset = self._dataset
         nframe = self._sv.getFrameNumber()
         self._sv.setStack(dataset.get_data(self.indices))
         self._sv.setFrameNumber(nframe)
@@ -485,3 +542,13 @@ class NoiseRemovalWidget(qt.QMainWindow):
     def bottom_threshold(self, bottom):
         self._bottom_threshold = bottom
         self._parametersDock.bottomLE.setText(bottom)
+
+    @property
+    def mask(self):
+        return self._mask
+
+    @mask.setter
+    def mask(self, mask):
+        self._mask = mask
+        if mask is not None:
+            self._sv.getPlotWidget().setSelectionMask(self._mask)
