@@ -124,6 +124,8 @@ class ROISelectionWidget(qt.QWidget):
         self._dataset = dataset
         self._update_dataset = dataset
         self.indices = indices
+        if self._dataset.title != "":
+            self._sv.setTitleCallback(lambda idx: self._dataset.title)
         self.setStack()
         self.bg_indices = bg_indices
         self.bg_dataset = bg_dataset
