@@ -67,11 +67,11 @@ class ShiftCorrectionDialog(qt.QDialog):
         resetB.clicked.connect(self.mainWindow.resetStack)
         self.mainWindow.computingSignal.connect(self._toggleButton)
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
 
         if dataset is not None:
             self._buttons.setEnabled(True)
-            self.mainWindow.setDataset(dataset, indices, bg_indices, bg_dataset)
+            self.mainWindow.setDataset(parent, dataset, indices, bg_indices, bg_dataset)
 
     def _toggleButton(self, state):
         self._buttons.button(qt.QDialogButtonBox.Ok).setEnabled(not state)
@@ -123,28 +123,35 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         self._chooseDimensionDock.widget.filterChanged.connect(self._filterStack)
         self._chooseDimensionDock.widget.stateDisabled.connect(self._wholeStack)
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
         Dataset setter. Saves the dataset and updates the stack with the dataset
         data
 
         :param Dataset dataset: dataset
         """
+        self.parent = parent
         self.dataset = dataset
         self._update_dataset = dataset
         self.indices = indices
         self.bg_indices = bg_indices
         self.bg_dataset = bg_dataset
         self._inputDock.widget.correctionB.setEnabled(True)
+        if self.dataset.title != "":
+            self._sv.setTitleCallback(lambda idx: self.dataset.title)
         if len(self.dataset.data.shape) > 3:
             self._chooseDimensionDock.show()
             self._chooseDimensionDock.widget.setDimensions(self._update_dataset.dims)
         if not self._chooseDimensionDock.widget._checkbox.isChecked():
             self._wholeStack()
 
-    def getDataset(self):
+    def getDataset(self, update=None):
         return self._update_dataset, self.indices, self.bg_indices, self.bg_dataset
 
+    def _updateDataset(self, widget, dataset):
+        self.parent._updateDataset(widget, dataset)
+        self.dataset = dataset
+
     def correct(self):
         """
         Function that starts the thread to compute the shift given
@@ -199,7 +206,13 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         if self._filtered_shift is None:
             self.shift = numpy.round(self.thread_detection.data[:, 1], 5)
         else:
-            self._filtered_shift = numpy.round(self.thread_detection.data[:, :, 1], 5)
+            self._filtered_shift = []
+            for s in self.thread_detection.data:
+                try:
+                    self._filtered_shift.append(numpy.round(s[:, 1], 5))
+                except (IndexError, TypeError):
+                    self._filtered_shift.append([0, 0])
+            self._filtered_shift = numpy.array(self._filtered_shift)
             self.shift = self._filtered_shift[self._dimension[1][0]]
         self.computingSignal.emit(False)
 
@@ -213,7 +226,10 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         self._inputDock.widget.correctionB.setEnabled(True)
         self.computingSignal.emit(False)
         if self.thread_correction.data:
+            del self._update_dataset
             self._update_dataset = self.thread_correction.data
+            self.thread_correction.data = None
+            self.thread_correction.func = None
             assert self._update_dataset is not None
             self.setStack(self._update_dataset)
         else:
@@ -229,7 +245,8 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         if dataset is None:
             dataset = self.dataset
         nframe = self._sv.getFrameNumber()
-        self._sv.setStack(dataset.get_data(self.indices, self._dimension) if dataset is not None else None)
+        data = dataset.get_data(self.indices, self._dimension) if dataset is not None else None
+        self._sv.setStack(data if len(data) else None)
         self._sv.setFrameNumber(nframe)
 
     def clearStack(self):
@@ -246,10 +263,8 @@ class ShiftCorrectionWidget(qt.QMainWindow):
             self._inputDock.widget.checkbox.show()
             self._filtered_shift = numpy.zeros((stack_size, 2)) if reset_shift else self._filtered_shift
             self.shift = self._filtered_shift[val[0]]
-        if data.shape[0]:
-            self._sv.setStack(data)
-        else:
-            self._sv.setStack(None)
+
+        self._sv.setStack(data if len(data) else None)
 
     def _wholeStack(self):
         self._dimension = None
@@ -342,6 +357,7 @@ class _InputWidget(qt.QWidget):
         Set the shift in the x axis
         """
         self.dxLE.setText(str(dx))
+        self.dxLE.setCursorPosition(0)
 
     def getDx(self):
         """
@@ -356,6 +372,7 @@ class _InputWidget(qt.QWidget):
         Set the shift in the x axis
         """
         self.dyLE.setText(str(dy))
+        self.dyLE.setCursorPosition(0)
 
     def getDy(self):
         """
