@@ -33,7 +33,7 @@ import h5py
 from datetime import datetime
 import numpy
 
-from silx.io.dictdump import dicttoh5
+from silx.io.dictdump import dicttoh5, h5todict
 
 _logger = logging.getLogger(__file__)
 
@@ -51,7 +51,7 @@ def assert_string(s, enums):
 
 
 # Print iterations progress
-def advancement_display(ncurrent, ntotal, prefix='', suffix='', decimals=1, length=100, fill='█', left="-"):
+def advancement_display(ncurrent, ntotal, prefix='', suffix='', decimals=1, length=100, fill='\u2588', left="-"):
     """
     Call in a loop to create terminal progress bar
     @params:
@@ -65,6 +65,7 @@ def advancement_display(ncurrent, ntotal, prefix='', suffix='', decimals=1, leng
         left       - Optional  : bar fill character (Str)
     """
     if ntotal:
+        ncurrent = min(ncurrent, ntotal)
         progress = ncurrent / float(ntotal)
     else:
         progress = ncurrent + 1
@@ -176,19 +177,21 @@ def read_components(h5_file):
         nx_entry = nx[nx.attrs["default"]]
         # find the default NXdata group
         nx_process = nx_entry["process_1"]
-        input_data = nx_process["inputs"]
-        dimensions = {}
+        input_data = nx_process["dimensions"]
+        dimensions = h5todict(h5_file, nx.attrs["default"] + "/process_1/dimensions",
+                              asarray=False)
+        input_data = nx_process["values"]
+        values = {}
         for key in input_data.keys():
-            dimensions[key] = numpy.array(list(input_data[key]))
+            values[key] = numpy.array(list(input_data[key]))
         results = nx_process["results"]
         components = numpy.array(list(results["components"]))
         W = numpy.array(list(results["W"]))
-        shape = list(results["shape"])
 
-    return dimensions, components, W, shape
+    return dimensions, components, W, values
 
 
-def write_components(h5_file, entry, dimensions, W, shape, data, processing_order,
+def write_components(h5_file, entry, dimensions, W, data, values, processing_order,
                      data_path='/', overwrite=True):
     """
     Write a stack of components and its parameters into .h5
@@ -252,10 +255,14 @@ def write_components(h5_file, entry, dimensions, W, shape, data, processing_orde
         nx_process['date'] = datetime.now().replace(microsecond=0).isoformat()
         nx_process['processing_order'] = numpy.int32(processing_order)
 
-        nx_parameters = nx_process.require_group("inputs")
+        nx_parameters = nx_process.require_group("dimensions")
         nx_parameters.attrs['NX_class'] = "NXparameters"
-        for key, value in dimensions.items():
-            save_key(nx_parameters.name, key_path=key, value=value)
+        dicttoh5(dimensions, h5f, entry + "/" + process_name + "/dimensions")
+
+        nx_values = nx_process.require_group("values")
+        nx_values.attrs["NX_class"] = "NXparameters"
+        for key, value in values.items():
+            save_key(nx_values.name, key_path=key, value=value)
 
         results = nx_process.require_group("results")
         results.attrs["NX_class"] = "NXcollection"
@@ -267,6 +274,33 @@ def write_components(h5_file, entry, dimensions, W, shape, data, processing_orde
         results.attrs["target"] = "components"
 
         save_key(results.name, "W", W)
-        save_key(results.name, "shape", shape)
         save_key(results.name, "components", data)
         save_key(nx_data.name, "components", h5f[source_addr])
+
+
+def create_nxdata_dict(signal, signal_name, axes=None, axes_names=None, axes_long_names=None, rgba=False):
+
+    nxdata = {
+        signal_name: signal,
+        "@signal": signal_name,
+        "@NX_class": "NXdata"
+    }
+
+    if axes is not None:
+
+        nxdata.update({
+            "@axes": axes_names,
+            axes_names[0]: axes[0],
+            axes_names[1]: axes[1]
+        })
+
+        if axes_long_names is not None:
+            nxdata.update({
+                axes_names[0] + "@long_name": axes_long_names[0],
+                axes_names[1] + "@long_name": axes_long_names[1]
+            })
+
+    if rgba:
+        nxdata[signal_name + "@interpretation"] = "rgba-image"
+
+    return nxdata
