@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "19/07/2019"
+__date__ = "15/04/2020"
 
 
 from numpy.distutils.misc_util import Configuration
@@ -38,7 +38,9 @@ def configuration(parent_package='', top_path=None):
     config.add_subpackage('gui')
     config.add_subpackage('test')
     config.add_subpackage('io')
-
+    config.add_subpackage('decomposition')
+    config.add_subpackage('resources')
+    config.add_subpackage("examples", "../examples")
     return config
 
 
