@@ -73,7 +73,7 @@ class BlindSourceSeparationWidgetOW(OWWidget):
 
     @Inputs.dataset
     def setDataset(self, _input):
-        try:
+        if _input is not None:
             dataset, update = _input
             self._widget.setDataset(*dataset)
 
@@ -81,8 +81,6 @@ class BlindSourceSeparationWidgetOW(OWWidget):
                 self.open()
             self.Outputs.dataset.send(((self,) + dataset[1:], update))
             self.Outputs.colormap.send(self._widget.getDisplayComponentsWidget().getStackViewColormap())
-        except Exception:
-            self.Outputs.dataset.send(None)
 
     @Inputs.colormap
     def setColormap(self, colormap):
