@@ -104,7 +104,7 @@ class DimensionWidgetOW(OWWidget):
         """
         Input signal to set the dataset.
         """
-        try:
+        if _input is not None:
             dataset, update = _input
             self.buttons.button(qt.QDialogButtonBox.Ok).setEnabled(False)
             try:
@@ -121,8 +121,6 @@ class DimensionWidgetOW(OWWidget):
                     self.open()
                 elif update != self:
                     self.Outputs.dataset.send(((self,) + self._widget.getDataset(), update))
-        except Exception:
-            self.Outputs.dataset.send(None)
 
     def _updateDataset(self, widget, dataset):
         self._widget._updateDataset(widget, dataset)
