@@ -28,6 +28,8 @@ __authors__ = ["J. Garriga"]
 __license__ = "MIT"
 __date__ = "11/12/2020"
 
+import numpy
+
 from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Input, Output
 from silx.gui.colors import Colormap
@@ -62,6 +64,7 @@ class NoiseRemovalWidgetOW(OWWidget):
     bottom_threshold = Setting(str())
     chunks = Setting(list())
     step = Setting(str())
+    mask = Setting(list())
 
     def __init__(self):
         super().__init__()
@@ -72,7 +75,7 @@ class NoiseRemovalWidgetOW(OWWidget):
 
     @Inputs.dataset
     def setDataset(self, _input):
-        try:
+        if _input is not None:
             dataset, update = _input
             self._widget.setDataset(*dataset)
             if self.method:
@@ -87,15 +90,13 @@ class NoiseRemovalWidgetOW(OWWidget):
                 self._widget.mainWindow.step = self.step
             if self.chunks:
                 self._widget.mainWindow.chunks = self.chunks
+            if self.mask:
+                self._widget.mainWindow.mask = self.mask
 
             if update is None:
                 self.show()
             elif update != self:
                 self.Outputs.dataset.send(((self,) + self._widget.mainWindow.getDataset(), update))
-        except Exception:
-            self._widget.mainWindow.clearStack()
-            # Emit None
-            self.Outputs.dataset.send(None)
 
     @Inputs.colormap
     def setColormap(self, colormap):
@@ -115,6 +116,10 @@ class NoiseRemovalWidgetOW(OWWidget):
         self.step = self._widget.mainWindow.step
         self.chunks = self._widget.mainWindow.chunks
         self.bottom_threshold = self._widget.mainWindow.bottom_threshold
+        if isinstance(self._widget.mainWindow.mask, numpy.ndarray):
+            self.mask = self._widget.mainWindow.mask.tolist()
+        else:
+            self.mask = self._widget.mainWindow.mask
         self.Outputs.dataset.send(((self,) + self._widget.mainWindow.getDataset(), None))
         self.Outputs.colormap.send(self._widget.mainWindow.getStackViewColormap())
         self.close()
