@@ -73,7 +73,7 @@ class RoiSelectionWidgetOW(OWWidget):
 
     @Inputs.dataset
     def setDataset(self, _input):
-        try:
+        if _input is not None:
             dataset, update = _input
             self._widget.setDataset(*dataset)
             # Set saved roi
@@ -84,10 +84,6 @@ class RoiSelectionWidgetOW(OWWidget):
                 self.open()
             elif update != self:
                 self.Outputs.dataset.send(((self,) + self._widget.getDataset(), update))
-        except Exception:
-            self._widget.clearStack()
-            # Emit None
-            self.Outputs.dataset.send(None)
 
     @Inputs.colormap
     def setColormap(self, colormap):
