@@ -70,7 +70,7 @@ class ShiftCorrectionWidgetOW(OWWidget):
 
     @Inputs.dataset
     def setDataset(self, _input):
-        try:
+        if _input is not None:
             dataset, update = _input
             self._widget.setDataset(*dataset)
             if len(self.shift):
@@ -80,10 +80,6 @@ class ShiftCorrectionWidgetOW(OWWidget):
                 self.open()
             elif update != self:
                 self.Outputs.dataset.send(((self,) + self._widget.mainWindow.getDataset(), update))
-        except Exception:
-            self._widget.mainWindow.clearStack()
-            # Emit None
-            self.Outputs.dataset.send(None)
 
     @Inputs.colormap
     def setColormap(self, colormap):
