@@ -58,15 +58,12 @@ class ZSumWidgetOW(OWWidget):
 
     @Inputs.dataset
     def setDataset(self, _input):
-        try:
+        if _input is not None:
             dataset, update = _input
             self._widget.setDataset(*dataset)
             if update is None:
                 self.open()
 
-        except Exception:
-            self._widget._plot.clear()
-
     @Inputs.colormap
     def setColormap(self, colormap):
         self._widget.setColormap(colormap)
